﻿using Ozeki.VoIP;
using System;
using System.Collections.Generic;
using System.Timers;
using Ozeki.Media;

namespace _07_IVR_database
{
    class Menu : ICommand
    {
        Dictionary<int, MultipleCommandHandler> _keys;
        MultipleCommandHandler _init;
        ICall _call;
        Timer _greetingMessageTimer;
        MultipleCommandHandler _handler;
        
        public Menu()
        {
            _keys = new Dictionary<int, MultipleCommandHandler>();
            _init = new MultipleCommandHandler();   
            _greetingMessageTimer = new Timer();
            _greetingMessageTimer.AutoReset = true;
        }

        void greetingMessageTimer_Elapsed(object sender, ElapsedEventArgs e)
        {
            _init.Cancel();
            _init.Start(_call);
        }

        public void Start(ICall call)
        {
            _call = call;
            Subscribe();
            _greetingMessageTimer.Interval = 20000;
            call.Answer();
            _greetingMessageTimer.Start();
            _init.Start(call);
        }

        void call_CallStateChanged(object sender, CallStateChangedArgs e)
        {
            
        }


        void call_DtmfReceived(object sender, VoIPEventArgs<DtmfInfo> e)
        {
            if(_keys.TryGetValue(e.Item.Signal.Signal, out _handler))
            {
                Unsubscribe();
                _handler.Completed += handler_Completed;
                _handler.Start(_call);

            }
        }

        public void Unsubscribe()
        {
            _init.Cancel();
            _greetingMessageTimer.Stop();

            if (_handler != null)
                _handler.Cancel();

            _call.CallStateChanged -= call_CallStateChanged;
            _call.DtmfReceived -= call_DtmfReceived;
            _greetingMessageTimer.Elapsed -= greetingMessageTimer_Elapsed;
        }

        private void Subscribe()
        {
            Unsubscribe();
            _call.CallStateChanged += call_CallStateChanged;
            _call.DtmfReceived += call_DtmfReceived;
            _greetingMessageTimer.Elapsed += greetingMessageTimer_Elapsed;
            _greetingMessageTimer.Start();
        }

        void handler_Completed(object sender, EventArgs e)
        {
            Subscribe();

            _init.Start(_call);
        }

        public void AddInitCommand(ICommand command)
        {
            _init.AddCommand(command);
        }

        public void AddKeypressCommand(int digit, ICommand command)
        {
            if (!_keys.ContainsKey(digit))
                _keys[digit] = new MultipleCommandHandler();

            _keys[digit].AddCommand(command);
        }

        public void Cancel()
        {
            
        }

        public event EventHandler Completed;
    }
}
